from fastapi import FastAPI
from .routing.Routing import Routing
from ..core import paths
from .. import __version__, config
import importlib
import fnc
import os


class Application(FastAPI):

    def bootstrap(self):
        self.title = fnc.get("docs.title", config, default="NOR/K")
        self.version = fnc.get("docs.version", config, default=__version__)

        self.routing()

        return self

    def routing(self):
        Routing.init(self)

        self.dynamic_routing()

    def dynamic_routing(self):
        if os.path.isdir(paths.ROUTES_PATH):
            for module in paths.list_dir(paths.ROUTES_PATH):
                importlib.import_module(f"routes.{module}")



from nork.core.logging import Log
from ..application import Application
import importlib

app = Application()
app.bootstrap()

try:
    server = importlib.import_module(f"server")
except Exception as exception:
    if type(exception).__name__ != "ModuleNotFoundError":
        Log.error(f"server.py :: {exception}")
    server = None


@app.on_event("startup")
async def on_startup():
    try:
        await getattr(server, 'on_startup')(app)
    except Exception as exception:
        if type(exception).__name__ != "AttributeError":
            Log.error(f"Event Startup :: {exception}")


@app.on_event("shutdown")
async def on_shutdown():
    try:
        await getattr(server, 'on_shutdown')(app)
    except Exception as exception:
        if type(exception).__name__ != "AttributeError":
            Log.error(f"Event Shutdown :: {exception}")




from typing import Union
import typer
import importlib
import sys

from .. import __version__, config
from ..core import paths


class Application:

    app: typer.Typer

    @classmethod
    def bootstrap(self):
        self.app = typer.Typer(
            help=f"NOR/K {typer.style(__version__, fg=typer.colors.GREEN)}")

        sys.path.append(paths.PROJECT_PATH)

        self.dynamic_commands(["OsEnv", "FrameworkNew", "Install"], "nork.commands")

        if config.get("framework"):
            self.dynamic_commands(["Serve"], "nork.framework.commands")

        self.dynamic_commands(paths.COMMANDS_PATH, "commands")
        self.dynamic_commands(paths.COMMANDS_PATH_APP, "app.commands")

        return self

    def dynamic_commands(path: Union[str, list], module_path: str):
        try:
            for module in (paths.list_dir(path) if isinstance(path, str) else path):
                importlib.import_module(f"{module_path}.{module}")
        except Exception as exception:
            pass



import typer


class Log:

    title: str = "NOR/K"

    @classmethod
    def error(self, msg: str):
        typer.echo(typer.style(
            f"[{self.title}]", fg=typer.colors.WHITE, bg=typer.colors.RED) + f" {msg}")



from nork.commands import nork
from ..core.logging import Log
from urllib.request import urlopen
from io import BytesIO
from zipfile import ZipFile
import os
import requests
import fnc


@nork.command(name="framework:new")
def handle(name: str, version: str = None):
    """
    Create a new NOR/K Framework project
    """
    try:
        if version is None:
            get_realeses = requests.get(
                "https://api.github.com/repos/getnork/framework/releases").json()

            for release in get_realeses:
                get_realese = release
                break
        else:
            get_realese = requests.get(
                f"https://api.github.com/repos/getnork/framework/releases/tags/v{version}").json()

        zipball_url = fnc.get("zipball_url", get_realese)
        version_tag = fnc.get("tag_name", get_realese)

        print(f'Creating application "{name}" version {version_tag}')

        with urlopen(zipball_url) as zip_response:
            with ZipFile(BytesIO(zip_response.read())) as zip_file:
                zip_file.extractall(os.getcwd())

        framework_directory = None

        for directory in os.listdir(os.getcwd()):
            if directory.startswith('getnork-framework-'):
                framework_directory = directory
                break

        if framework_directory is not None:
            os.rename(f"{os.getcwd()}/{framework_directory}",
                      f"{os.getcwd()}/{name}")

        print(framework_directory, version_tag, zipball_url)
    except Exception as exception:
        Log.error(exception)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nork',
 'nork.cli',
 'nork.commands',
 'nork.core',
 'nork.framework',
 'nork.framework.commands',
 'nork.framework.routing',
 'nork.framework.server']

package_data = \
{'': ['*']}

install_requires = \
['fnc>=0.5.2,<0.6.0',
 'python-dotenv>=0.18.0,<0.19.0',
 'requests>=2.25.1,<3.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer[all]>=0.3.2,<0.4.0']

extras_require = \
{'framework': ['fastapi[all]>=0.65.2,<0.66.0']}

entry_points = \
{'console_scripts': ['nork = nork.cli:app']}

setup_kwargs = {
    'name': 'nork',
    'version': '0.1.15',
    'description': 'CLI for open source PaaS and framework.',
    'long_description': '# NOR/K\nCLI for open source PaaS and framework.\n\n## Installation\nInstallation is automated with the following command:\n\n```bash\n/bin/bash -c "$(curl -fsSL https://nork.link/CLI-install)"\n```\n',
    'author': 'NORK',
    'author_email': 'support@nork.sh',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://nork.sh',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import os

PROJECT_PATH = os.getcwd()
APP_PATH = os.path.join(PROJECT_PATH, 'app')
ROUTES_PATH = os.path.join(PROJECT_PATH, 'routes')
COMMANDS_PATH = os.path.join(PROJECT_PATH, 'commands')
COMMANDS_PATH_APP = os.path.join(PROJECT_PATH, 'app', 'commands')


def list_dir(dir: str):
    modules = [os.path.splitext(_file)[0] for _file in os.listdir(
        dir) if not _file.startswith('__') and not _file.startswith('.')]

    return modules



from nork.commands import nork
from subprocess import call
import os


if os.path.isfile('requirements.txt'):
    @nork.command(name="install")
    def handle():
        """
        Install all of requirements.txt
        """
        try:
            call(["pip", "install", "-r", "requirements.txt"])
        except Exception:
            call(["pip3", "install", "-r", "requirements.txt"])



from nork.commands import nork
from nork import config
import fnc
import uvicorn


@nork.command(name="serve")
def handle(host: str = "0.0.0.0", port: int = 8080, reload: bool = True):
    """
    Serve the application on the development server
    """
    host = fnc.get("server.host", config, default=host)
    port = fnc.get("server.port", config, default=port)
    reload = fnc.get("server.reload", config, default=reload)

    if fnc.get("server.custom", config):
        uvicorn.run(fnc.get("server.module", config,
                    default="server:app"), host=host, port=port, reload=reload)
    else:
        uvicorn.run("nork.framework.server:app",
                    host=host, port=port, reload=reload)



from types import FunctionType
from typing import Any
from fastapi.applications import FastAPI
from nork.core.logging import Log
import importlib


class Routing:

    app: FastAPI

    @classmethod
    def init(self, app):
        self.app = app

        return self

    def add_route(self, path: str, controller: Any, method: str = "GET", **kwargs):
        constroller = self._import_controller(controller)

        if constroller is not None:
            self.app.add_api_route(path=path, methods=[
                                   method], endpoint=constroller, **kwargs)

        return self

    def _import_controller(self, endpoint: Any):
        if isinstance(endpoint, FunctionType):
            return endpoint
        elif isinstance(endpoint, str):
            try:
                endpoints = endpoint.split("@")
                controller_dirs = endpoints[0].split("\\")
                controller_dirs_dot = ".".join(controller_dirs)
                controller_name = controller_dirs[-1]
                method_name = endpoints[-1]

                module = importlib.import_module(
                    f"app.controllers.{controller_dirs_dot}")
                controller = getattr(module, controller_name)

                return getattr(controller, method_name)
            except Exception as e:
                Log.error(f"Routing: {str(e)}")

                return None
        else:
            return None



from typing import Any
from .Routing import Routing

routing = Routing()


class Route:

    @classmethod
    def get(self, path, controller=None, **kwargs: Any):
        return routing.add_route(path=path, method="GET", controller=controller, **kwargs)

    @classmethod
    def post(self, path, controller=None, **kwargs: Any):
        return routing.add_route(path=path, method="POST", controller=controller, **kwargs)

    @classmethod
    def put(self, path, controller=None, **kwargs: Any):
        return routing.add_route(path=path, method="PUT", controller=controller, **kwargs)
